//******************************************************************************
//
// File:    NewSurveyDialog.java
// Package: edu.rit.survey
// Unit:    Class edu.rit.survey.NewSurveyDialog
//
// This Java source file is copyright (C) 2006 by Alan Kaminsky. All rights
// reserved. For further information, contact the author, Alan Kaminsky, at
// ark@cs.rit.edu.
//
// This Java source file is part of the M2MI Library ("The Library"). The
// Library is free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// The Library is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
// details.
//
// A copy of the GNU General Public License is provided in the file gpl.txt. You
// may also obtain a copy of the GNU General Public License on the World Wide
// Web at http://www.gnu.org/licenses/gpl.html or by writing to the Free
// Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA.
//
//******************************************************************************

package edu.rit.survey;

import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

/**
 * Class NewSurveyDialog provides a dialog for creating a new survey.
 *
 * @author  Alan Kaminsky
 * @version 24-Mar-2006
 */
public class NewSurveyDialog
	extends JDialog
	{

// Hidden constants.

	private static final int GAP = 10;

// Hidden data members.

	private NewSurveyDialog myself;

	private JTextField myName;
	private JTextField myQuestion;
	private JTextField[] myAnswer = new JTextField [SurveyState.NUM_ANSWERS];
	private JButton myOKButton;
	private JButton myCancelButton;

	private int myState;
		private static final int SHOWING = 0;
		private static final int OK_CLICKED = 1;
		private static final int CANCEL_CLICKED = 2;

// Exported constructors.

	/**
	 * Construct a new new survey dialog.
	 *
	 * @param  owner  Owner frame.
	 */
	public NewSurveyDialog
		(Frame owner)
		{
		super (owner, "New Survey", true);
		myself = this;

		JLabel label;
		GridBagConstraints constraints;

		JPanel panela = new JPanel();
		add (panela);
		panela.setBorder (BorderFactory.createEmptyBorder (GAP, GAP, GAP, GAP));
		panela.setLayout (new BoxLayout (panela, BoxLayout.Y_AXIS));

		JPanel panelb = new JPanel();
		panela.add (panelb);
		GridBagLayout layout = new GridBagLayout();
		panelb.setLayout (layout);

		label = new JLabel ("Name:  ");
		panelb.add (label);
		constraints = new GridBagConstraints();
		constraints.gridx = 0;
		constraints.gridy = 0;
		constraints.anchor = GridBagConstraints.WEST;
		layout.setConstraints (label, constraints);

		myName = new JTextField (40);
		panelb.add (myName);
		constraints = new GridBagConstraints();
		constraints.gridx = 1;
		constraints.gridy = 0;
		layout.setConstraints (myName, constraints);

		label = new JLabel ("Question:  ");
		panelb.add (label);
		constraints = new GridBagConstraints();
		constraints.gridx = 0;
		constraints.gridy = 1;
		constraints.anchor = GridBagConstraints.WEST;
		layout.setConstraints (label, constraints);

		myQuestion = new JTextField (40);
		panelb.add (myQuestion);
		constraints = new GridBagConstraints();
		constraints.gridx = 1;
		constraints.gridy = 1;
		layout.setConstraints (myQuestion, constraints);

		for (int i = 0; i < SurveyState.NUM_ANSWERS; ++ i)
			{
			label = new JLabel ("Answer " + (i+1) + ":  ");
			panelb.add (label);
			constraints = new GridBagConstraints();
			constraints.gridx = 0;
			constraints.gridy = i+2;
			constraints.anchor = GridBagConstraints.WEST;
			layout.setConstraints (label, constraints);

			myAnswer[i] = new JTextField (40);
			panelb.add (myAnswer[i]);
			constraints = new GridBagConstraints();
			constraints.gridx = 1;
			constraints.gridy = i+2;
			layout.setConstraints (myAnswer[i], constraints);
			}

		panela.add (Box.createVerticalStrut (GAP));

		panelb = new JPanel();
		panela.add (panelb);
		panelb.setLayout (new BoxLayout (panelb, BoxLayout.X_AXIS));

		myOKButton = new JButton ("OK");
		panelb.add (myOKButton);
		getRootPane().setDefaultButton (myOKButton);
		myOKButton.addActionListener (new ActionListener()
			{
			public void actionPerformed
				(ActionEvent event)
				{
				synchronized (myself)
					{
					setVisible (false);
					myState = OK_CLICKED;
					myself.notifyAll();
					}
				}
			});

		myCancelButton = new JButton ("Cancel");
		panelb.add (myCancelButton);
		myCancelButton.addActionListener (new ActionListener()
			{
			public void actionPerformed
				(ActionEvent event)
				{
				synchronized (myself)
					{
					setVisible (false);
					myState = CANCEL_CLICKED;
					myself.notifyAll();
					}
				}
			});

		addWindowListener (new WindowAdapter()
			{
			public void windowClosing
				(WindowEvent event)
				{
				synchronized (myself)
					{
					setVisible (false);
					myState = CANCEL_CLICKED;
					myself.notifyAll();
					}
				}
			});

		pack();
		}

// Exported operations.

	/**
	 * Display this new survey dialog and get a survey from the user.
	 *
	 * @return  Survey state if "OK" was clicked, null if "Cancel" was clicked.
	 */
	public SurveyState getSurveyState()
		{
		// Clear fields.
		myName.setText ("");
		myQuestion.setText ("");
		for (int i = 0; i < SurveyState.NUM_ANSWERS; ++ i)
			{
			myAnswer[i].setText ("");
			}
		myName.requestFocusInWindow();

		// Show dialog.
		setLocation (getOwner().getLocationOnScreen());
		myState = SHOWING;
		setVisible (true);

		// Wait until dialog is hidden.
		synchronized (myself)
			{
			while (myState == SHOWING)
				{
				try
					{
					myself.wait();
					}
				catch (InterruptedException exc)
					{
					}
				}
			}

		// Return survey state, or null.
		if (myState == OK_CLICKED)
			{
			return new SurveyState
				(myName.getText(),
				 myQuestion.getText(),
				 new String[]
					{myAnswer[0].getText(),
					 myAnswer[1].getText(),
					 myAnswer[2].getText(),
					 myAnswer[3].getText()});
			}
		else
			{
			return null;
			}
		}

	}
